/* $Id$ */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xcms.h>

#include <assert.h>
#include <err.h>
#include <stdlib.h>

#define WIDTH 640
#define HEIGHT 480

unsigned long pixels[WIDTH];

void
alloc_palette(Display *display, Colormap cmap)
{
	double step, angle;
	XcmsColor cms;
	int i;

	step = 360.0 / WIDTH;
	for (i = 0; i < WIDTH; i++) {
		angle = i * step;
		cms.spec.TekHVC.H = angle;
		cms.spec.TekHVC.V = 75.0;
		cms.spec.TekHVC.C = 75.0;
		cms.format = XcmsTekHVCFormat;
		XcmsAllocColor(display, cmap, &cms, XcmsRGBFormat);
		pixels[i] = cms.pixel;
	}
}

void
draw(Display *display, Drawable drawable, GC gc, int x, int y)
{
	int i;

	for (i = 0; i < x; i++) {
		XSetForeground(display, gc, pixels[i]);
		XDrawLine(display, drawable, gc,
				i, 0, i, y);
	}
	XFlush(display);
}

int
main()
{
	Display *display;
	Window root, window;
	Colormap colormap;
	XEvent event;
	Atom delete;
	GC gc;
	int screen;
	int die = 0;
	unsigned long white, black;

	display = XOpenDisplay(NULL);
	assert(display);

	screen = DefaultScreen(display);
	colormap = DefaultColormap(display, screen);
	root = RootWindow(display, screen);
	white = WhitePixel(display, screen);
	black = BlackPixel(display, screen);
	gc = XDefaultGC(display, screen);

	alloc_palette(display, colormap);

	window = XCreateSimpleWindow(display, root,
			0, 0, WIDTH, HEIGHT, 0, black, white);
	XSelectInput(display, window, StructureNotifyMask);
	delete = XInternAtom(display, "WM_DELETE_WINDOW", 0);
	XSetWMProtocols(display, window, &delete, 1);
	XMapWindow(display, window);

	while (!die) {
		XNextEvent(display, &event);
		switch (event.type) {
		case ClientMessage:
			die = *event.xclient.data.l == delete;
			break;
		default:
			draw(display, window, gc, WIDTH, HEIGHT);
			break;
		}
	}

	XUnmapWindow(display, window);
	XDestroyWindow(display, window);
	XCloseDisplay(display);

	return 0;
}
